@@ -1,4 +1,4 @@
-/* $OpenBSD: v3_pmaps.c,v 1.4 2014/07/11 08:44:49 jsing Exp $ */
+/* $OpenBSD: v3_pmaps.c,v 1.6 2015/02/10 05:43:09 jsing Exp $ */
 /* Written by Dr Stephen N Henson (steve@openssl.org) for the OpenSSL
  * project.
  */
@@ -90,7 +90,18 @@ ASN1_EX_TEMPLATE_TYPE(ASN1_TFLG_SEQUENCE_OF, 0, POLICY_MAPPINGS,
     POLICY_MAPPING)
 ASN1_ITEM_TEMPLATE_END(POLICY_MAPPINGS)
 
-IMPLEMENT_ASN1_ALLOC_FUNCTIONS(POLICY_MAPPING)
+
+POLICY_MAPPING *
+POLICY_MAPPING_new(void)
+{
+	return (POLICY_MAPPING*)ASN1_item_new(&POLICY_MAPPING_it);
+}
+
+void
+POLICY_MAPPING_free(POLICY_MAPPING *a)
+{
+	ASN1_item_free((ASN1_VALUE *)a, &POLICY_MAPPING_it);
+}
 
 static STACK_OF(CONF_VALUE) *
 i2v_POLICY_MAPPINGS(const X509V3_EXT_METHOD *method, void *a,
@@ -115,9 +126,9 @@ static void *
 v2i_POLICY_MAPPINGS(const X509V3_EXT_METHOD *method, X509V3_CTX *ctx,
     STACK_OF(CONF_VALUE) *nval)
 {
-	POLICY_MAPPINGS *pmaps;
-	POLICY_MAPPING *pmap;
-	ASN1_OBJECT *obj1, *obj2;
+	POLICY_MAPPINGS *pmaps = NULL;
+	POLICY_MAPPING *pmap = NULL;
+	ASN1_OBJECT *obj1 = NULL, *obj2 = NULL;
 	CONF_VALUE *val;
 	int i, rc;
 
@@ -145,10 +156,12 @@ v2i_POLICY_MAPPINGS(const X509V3_EXT_METHOD *method, X509V3_CTX *ctx,
 		}
 		pmap->issuerDomainPolicy = obj1;
 		pmap->subjectDomainPolicy = obj2;
+		obj1 = obj2 = NULL;
 		if (sk_POLICY_MAPPING_push(pmaps, pmap) == 0) {
 	    		rc = ERR_R_MALLOC_FAILURE;
 			goto err;
 		}
+		pmap = NULL;
 	}
 	return pmaps;
 
@@ -157,5 +170,8 @@ err:
 	X509V3err(X509V3_F_V2I_POLICY_MAPPINGS, rc);
 	if (rc == X509V3_R_INVALID_OBJECT_IDENTIFIER)
 		X509V3_conf_err(val);
+	ASN1_OBJECT_free(obj1);
+	ASN1_OBJECT_free(obj2);
+	POLICY_MAPPING_free(pmap);
 	return NULL;
 }
